@@ -40,7 +40,7 @@
 #include "nsContentCID.h"
 #include "nsDOMError.h"
 #include "nsIChannel.h"
-#include "nsIContent.h"
+#include "mozilla/dom/Element.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMText.h"
 #include "nsIDocument.h"
@@ -67,6 +67,9 @@
 #include "txExprParser.h"
 #include "nsIErrorService.h"
 #include "nsIScriptSecurityManager.h"
+#include "nsJSUtils.h"
+
+using namespace mozilla::dom;
 
 static NS_DEFINE_CID(kXMLDocumentCID, NS_XMLDOCUMENT_CID);
 
@@ -78,10 +81,8 @@ class txToDocHandlerFactory : public txAOutputHandlerFactory
 public:
     txToDocHandlerFactory(txExecutionState* aEs,
                           nsIDOMDocument* aSourceDocument,
-                          nsIDOMDocument* aResultDocument,
                           nsITransformObserver* aObserver)
-        : mEs(aEs), mSourceDocument(aSourceDocument),
-          mResultDocument(aResultDocument), mObserver(aObserver)
+        : mEs(aEs), mSourceDocument(aSourceDocument), mObserver(aObserver)
     {
     }
 
@@ -90,7 +91,6 @@ public:
 private:
     txExecutionState* mEs;
     nsCOMPtr<nsIDOMDocument> mSourceDocument;
-    nsCOMPtr<nsIDOMDocument> mResultDocument;
     nsCOMPtr<nsITransformObserver> mObserver;
 };
 
@@ -118,28 +118,41 @@ txToDocHandlerFactory::createHandlerWith(txOutputFormat* aFormat,
         case eXMLOutput:
         {
             *aHandler = new txUnknownHandler(mEs);
-            break;
+            return NS_OK;
         }
 
         case eHTMLOutput:
         {
-            *aHandler = new txMozillaXMLOutput(EmptyString(),
-                                               kNameSpaceID_None,
-                                               aFormat, mSourceDocument,
-                                               mResultDocument, mObserver);
-            break;
+            nsAutoPtr<txMozillaXMLOutput> handler(
+                new txMozillaXMLOutput(aFormat, mObserver));
+
+            nsresult rv = handler->createResultDocument(EmptyString(),
+                                                        kNameSpaceID_None,
+                                                        mSourceDocument);
+            if (NS_SUCCEEDED(rv)) {
+                *aHandler = handler.forget();
+            }
+
+            return rv;
         }
 
         case eTextOutput:
         {
-            *aHandler = new txMozillaTextOutput(mSourceDocument,
-                                                mResultDocument,
-                                                mObserver);
-            break;
+            nsAutoPtr<txMozillaTextOutput> handler(
+                new txMozillaTextOutput(mObserver));
+
+            nsresult rv = handler->createResultDocument(mSourceDocument);
+            if (NS_SUCCEEDED(rv)) {
+                *aHandler = handler.forget();
+            }
+
+            return rv;
         }
     }
-    NS_ENSURE_TRUE(*aHandler, NS_ERROR_OUT_OF_MEMORY);
-    return NS_OK;
+
+    NS_RUNTIMEABORT("Unknown output method");
+
+    return NS_ERROR_FAILURE;
 }
 
 nsresult
@@ -159,23 +172,35 @@ txToDocHandlerFactory::createHandlerWith(txOutputFormat* aFormat,
         case eXMLOutput:
         case eHTMLOutput:
         {
-            *aHandler = new txMozillaXMLOutput(aName, aNsID, aFormat,
-                                               mSourceDocument,
-                                               mResultDocument,
-                                               mObserver);
-            break;
+            nsAutoPtr<txMozillaXMLOutput> handler(
+                new txMozillaXMLOutput(aFormat, mObserver));
+
+            nsresult rv = handler->createResultDocument(aName, aNsID,
+                                                        mSourceDocument);
+            if (NS_SUCCEEDED(rv)) {
+                *aHandler = handler.forget();
+            }
+
+            return rv;
         }
 
         case eTextOutput:
         {
-            *aHandler = new txMozillaTextOutput(mSourceDocument,
-                                                mResultDocument,
-                                                mObserver);
-            break;
+            nsAutoPtr<txMozillaTextOutput> handler(
+                new txMozillaTextOutput(mObserver));
+
+            nsresult rv = handler->createResultDocument(mSourceDocument);
+            if (NS_SUCCEEDED(rv)) {
+                *aHandler = handler.forget();
+            }
+
+            return rv;
         }
     }
-    NS_ENSURE_TRUE(*aHandler, NS_ERROR_OUT_OF_MEMORY);
-    return NS_OK;
+
+    NS_RUNTIMEABORT("Unknown output method");
+
+    return NS_ERROR_FAILURE;
 }
 
 nsresult
@@ -193,10 +218,10 @@ txToFragmentHandlerFactory::createHandlerWith(txOutputFormat* aFormat,
             NS_ASSERTION(domdoc, "unable to get ownerdocument");
             nsCOMPtr<nsIDocument> doc = do_QueryInterface(domdoc);
 
-            if (!doc || doc->IsCaseSensitive()) {
-                format.mMethod = eXMLOutput;
-            } else {
+            if (doc && doc->IsHTML()) {
                 format.mMethod = eHTMLOutput;
+            } else {
+                format.mMethod = eXMLOutput;
             }
 
             *aHandler = new txMozillaXMLOutput(&format, mFragment, PR_FALSE);
@@ -311,15 +336,17 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_IMPL_CYCLE_COLLECTING_ADDREF(txMozillaXSLTProcessor)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(txMozillaXSLTProcessor)
+
+DOMCI_DATA(XSLTProcessor, txMozillaXSLTProcessor)
+
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(txMozillaXSLTProcessor)
     NS_INTERFACE_MAP_ENTRY(nsIXSLTProcessor)
-    NS_INTERFACE_MAP_ENTRY(nsIXSLTProcessorObsolete)
     NS_INTERFACE_MAP_ENTRY(nsIXSLTProcessorPrivate)
     NS_INTERFACE_MAP_ENTRY(nsIDocumentTransformer)
     NS_INTERFACE_MAP_ENTRY(nsIMutationObserver)
     NS_INTERFACE_MAP_ENTRY(nsIJSNativeInitializer)
     NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIXSLTProcessor)
-    NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(XSLTProcessor)
+    NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(XSLTProcessor)
 NS_INTERFACE_MAP_END
 
 txMozillaXSLTProcessor::txMozillaXSLTProcessor() : mStylesheetDocument(nsnull),
@@ -336,39 +363,6 @@ txMozillaXSLTProcessor::~txMozillaXSLTProcessor()
     }
 }
 
-NS_IMETHODIMP
-txMozillaXSLTProcessor::TransformDocument(nsIDOMNode* aSourceDOM,
-                                          nsIDOMNode* aStyleDOM,
-                                          nsIDOMDocument* aOutputDoc,
-                                          nsISupports* aObserver)
-{
-    NS_ENSURE_ARG(aSourceDOM);
-    NS_ENSURE_ARG(aStyleDOM);
-    NS_ENSURE_ARG(aOutputDoc);
-    NS_ENSURE_FALSE(aObserver, NS_ERROR_NOT_IMPLEMENTED);
-
-    if (!nsContentUtils::CanCallerAccess(aSourceDOM) ||
-        !nsContentUtils::CanCallerAccess(aStyleDOM) ||
-        !nsContentUtils::CanCallerAccess(aOutputDoc)) {
-        return NS_ERROR_DOM_SECURITY_ERR;
-    }
-
-    PRUint16 type = 0;
-    aStyleDOM->GetNodeType(&type);
-    NS_ENSURE_TRUE(type == nsIDOMNode::ELEMENT_NODE ||
-                   type == nsIDOMNode::DOCUMENT_NODE,
-                   NS_ERROR_INVALID_ARG);
-
-    nsCOMPtr<nsINode> styleNode = do_QueryInterface(aStyleDOM);
-    nsresult rv = TX_CompileStylesheet(styleNode, this, mPrincipal,
-                                       getter_AddRefs(mStylesheet));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    mSource = aSourceDOM;
-
-    return TransformToDoc(aOutputDoc, nsnull);
-}
-
 NS_IMETHODIMP
 txMozillaXSLTProcessor::SetTransformObserver(nsITransformObserver* aObserver)
 {
@@ -561,7 +555,7 @@ public:
 
   NS_IMETHOD Run()
   {
-    mProcessor->TransformToDoc(nsnull, nsnull);
+    mProcessor->TransformToDoc(nsnull);
     return NS_OK;
   }
 };
@@ -610,7 +604,7 @@ txMozillaXSLTProcessor::ImportStylesheet(nsIDOMNode *aStyle)
     
     nsCOMPtr<nsINode> styleNode = do_QueryInterface(aStyle);
     NS_ENSURE_TRUE(styleNode &&
-                   (styleNode->IsNodeOfType(nsINode::eELEMENT) ||
+                   (styleNode->IsElement() ||
                     styleNode->IsNodeOfType(nsINode::eDOCUMENT)),
                    NS_ERROR_INVALID_ARG);
 
@@ -619,7 +613,7 @@ txMozillaXSLTProcessor::ImportStylesheet(nsIDOMNode *aStyle)
     // XXX set up exception context, bug 204658
     NS_ENSURE_SUCCESS(rv, rv);
 
-    if (styleNode->IsNodeOfType(nsINode::eELEMENT)) {
+    if (styleNode->IsElement()) {
         mStylesheetDocument = styleNode->GetOwnerDoc();
         NS_ENSURE_TRUE(mStylesheetDocument, NS_ERROR_UNEXPECTED);
 
@@ -651,12 +645,11 @@ txMozillaXSLTProcessor::TransformToDocument(nsIDOMNode *aSource,
 
     mSource = aSource;
 
-    return TransformToDoc(nsnull, aResult);
+    return TransformToDoc(aResult);
 }
 
 nsresult
-txMozillaXSLTProcessor::TransformToDoc(nsIDOMDocument *aOutputDoc,
-                                       nsIDOMDocument **aResult)
+txMozillaXSLTProcessor::TransformToDoc(nsIDOMDocument **aResult)
 {
     nsAutoPtr<txXPathNode> sourceNode(txXPathNativeNode::createXPathNode(mSource));
     if (!sourceNode) {
@@ -673,14 +666,15 @@ txMozillaXSLTProcessor::TransformToDoc(nsIDOMDocument *aOutputDoc,
 
     // XXX Need to add error observers
 
-    txToDocHandlerFactory handlerFactory(&es, sourceDOMDocument, aOutputDoc,
-                                         mObserver);
+    txToDocHandlerFactory handlerFactory(&es, sourceDOMDocument, mObserver);
     es.mOutputHandlerFactory = &handlerFactory;
 
-    es.init(*sourceNode, &mVariables);
+    nsresult rv = es.init(*sourceNode, &mVariables);
 
     // Process root of XML source document
-    nsresult rv = txXSLTProcessor::execute(es);
+    if (NS_SUCCEEDED(rv)) {
+        rv = txXSLTProcessor::execute(es);
+    }
     
     nsresult endRv = es.end(rv);
     if (NS_SUCCEEDED(rv)) {
@@ -736,10 +730,12 @@ txMozillaXSLTProcessor::TransformToFragment(nsIDOMNode *aSource,
     txToFragmentHandlerFactory handlerFactory(*aResult);
     es.mOutputHandlerFactory = &handlerFactory;
 
-    es.init(*sourceNode, &mVariables);
+    rv = es.init(*sourceNode, &mVariables);
 
     // Process root of XML source document
-    rv = txXSLTProcessor::execute(es);
+    if (NS_SUCCEEDED(rv)) {
+        rv = txXSLTProcessor::execute(es);
+    }
     // XXX setup exception context, bug 204658
     nsresult endRv = es.end(rv);
     if (NS_SUCCEEDED(rv)) {
@@ -914,7 +910,7 @@ txMozillaXSLTProcessor::SetParameter(const nsAString & aNamespaceURI,
 
                 if (NS_FAILED(rv)) {
                     while (i < count) {
-                        NS_RELEASE(values[i]);
+                        NS_IF_RELEASE(values[i]);
                         ++i;
                     }
                     nsMemory::Free(array);
@@ -1077,7 +1073,7 @@ txMozillaXSLTProcessor::reportError(nsresult aResult,
     }
     else {
         nsCOMPtr<nsIStringBundleService> sbs =
-            do_GetService(NS_STRINGBUNDLE_CONTRACTID);
+            mozilla::services::GetStringBundleService();
         if (sbs) {
             nsXPIDLString errorText;
             sbs->FormatStatusMessage(aResult, EmptyString().get(),
@@ -1205,6 +1201,7 @@ txMozillaXSLTProcessor::ensureStylesheet()
 void
 txMozillaXSLTProcessor::NodeWillBeDestroyed(const nsINode* aNode)
 {
+    nsCOMPtr<nsIMutationObserver> kungFuDeathGrip(this);
     if (NS_FAILED(mCompileResult)) {
         return;
     }
@@ -1224,11 +1221,10 @@ txMozillaXSLTProcessor::CharacterDataChanged(nsIDocument* aDocument,
 
 void
 txMozillaXSLTProcessor::AttributeChanged(nsIDocument* aDocument,
-                                         nsIContent* aContent,
+                                         Element* aElement,
                                          PRInt32 aNameSpaceID,
                                          nsIAtom* aAttribute,
-                                         PRInt32 aModType,
-                                         PRUint32 aStateMask)
+                                         PRInt32 aModType)
 {
     mStylesheet = nsnull;
 }
@@ -1236,7 +1232,8 @@ txMozillaXSLTProcessor::AttributeChanged(nsIDocument* aDocument,
 void
 txMozillaXSLTProcessor::ContentAppended(nsIDocument* aDocument,
                                         nsIContent* aContainer,
-                                        PRInt32 aNewIndexInContainer)
+                                        nsIContent* aFirstNewContent,
+                                        PRInt32 /* unused */)
 {
     mStylesheet = nsnull;
 }
@@ -1245,7 +1242,7 @@ void
 txMozillaXSLTProcessor::ContentInserted(nsIDocument* aDocument,
                                         nsIContent* aContainer,
                                         nsIContent* aChild,
-                                        PRInt32 aIndexInContainer)
+                                        PRInt32 /* unused */)
 {
     mStylesheet = nsnull;
 }
@@ -1254,7 +1251,8 @@ void
 txMozillaXSLTProcessor::ContentRemoved(nsIDocument* aDocument,
                                        nsIContent* aContainer,
                                        nsIContent* aChild,
-                                       PRInt32 aIndexInContainer)
+                                       PRInt32 aIndexInContainer,
+                                       nsIContent* aPreviousSibling)
 {
     mStylesheet = nsnull;
 }
@@ -1458,10 +1456,8 @@ txVariable::Convert(nsIVariant *aValue, txAExprResult** aResult)
                 JSString *str = JS_ValueToString(cx, OBJECT_TO_JSVAL(jsobj));
                 NS_ENSURE_TRUE(str, NS_ERROR_FAILURE);
 
-                const PRUnichar *strChars =
-                    reinterpret_cast<const PRUnichar*>
-                                    (::JS_GetStringChars(str));
-                nsDependentString value(strChars, ::JS_GetStringLength(str));
+                nsDependentJSString value;
+                NS_ENSURE_TRUE(value.init(cx, str), NS_ERROR_FAILURE);
 
                 *aResult = new StringResult(value, nsnull);
                 if (!*aResult) {
