@@ -48,6 +48,7 @@
 #include "nsDOMClassInfo.h"
 #include "nsPluginError.h"
 #include "nsContentUtils.h"
+#include "nsPluginHost.h"
 
 nsPluginArray::nsPluginArray(nsNavigator* navigator,
                              nsIDocShell *aDocShell)
@@ -70,6 +71,8 @@ nsPluginArray::~nsPluginArray()
   }
 }
 
+DOMCI_DATA(PluginArray, nsPluginArray)
+
 // QueryInterface implementation for nsPluginArray
 NS_INTERFACE_MAP_BEGIN(nsPluginArray)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIDOMPluginArray)
@@ -83,8 +86,9 @@ NS_IMPL_RELEASE(nsPluginArray)
 NS_IMETHODIMP
 nsPluginArray::GetLength(PRUint32* aLength)
 {
-  if (AllowPlugins() && mPluginHost)
-    return mPluginHost->GetPluginCount(aLength);
+  nsPluginHost *pluginHost = static_cast<nsPluginHost*>(mPluginHost.get());
+  if (AllowPlugins() && pluginHost)
+    return pluginHost->GetPluginCount(aLength);
   
   *aLength = 0;
   return NS_OK;
@@ -187,11 +191,18 @@ nsPluginArray::GetPluginHost(nsIPluginHost** aPluginHost)
 }
 
 void
-nsPluginArray::SetDocShell(nsIDocShell* aDocShell)
+nsPluginArray::SetDocShell(nsIDocShell *aDocShell)
 {
   mDocShell = aDocShell;
 }
 
+void
+nsPluginArray::Invalidate()
+{
+  mDocShell = nsnull;
+  mNavigator = nsnull;
+}
+
 NS_IMETHODIMP
 nsPluginArray::Refresh(PRBool aReloadDocuments)
 {
@@ -251,7 +262,8 @@ nsPluginArray::GetPlugins()
     if (!mPluginCount)
       return NS_OK;
 
-    rv = mPluginHost->GetPlugins(mPluginCount, mPluginArray);
+    nsPluginHost *pluginHost = static_cast<nsPluginHost*>(mPluginHost.get());
+    rv = pluginHost->GetPlugins(mPluginCount, mPluginArray);
     if (NS_SUCCEEDED(rv)) {
       // need to wrap each of these with a nsPluginElement, which
       // is scriptable.
@@ -297,6 +309,8 @@ nsPluginElement::~nsPluginElement()
 }
 
 
+DOMCI_DATA(Plugin, nsPluginElement)
+
 // QueryInterface implementation for nsPluginElement
 NS_INTERFACE_MAP_BEGIN(nsPluginElement)
   NS_INTERFACE_MAP_ENTRY(nsISupports)
