@@ -51,9 +51,10 @@
 #include "jsapi.h"
 #include "jscntxt.h"
 #include "jsversion.h"
-#include "jsdbgapi.h"
 #include "jsexn.h"
 #include "jsfun.h"
+#include "jsgc.h"
+#include "jsgcmark.h"
 #include "jsinterp.h"
 #include "jsnum.h"
 #include "jsobj.h"
@@ -63,10 +64,10 @@
 #include "jsstaticcheck.h"
 #include "jswrapper.h"
 
-#include "jscntxtinlines.h"
-#include "jsinterpinlines.h"
 #include "jsobjinlines.h"
 
+#include "vm/Stack-inl.h"
+
 using namespace js;
 using namespace js::gc;
 
@@ -80,22 +81,19 @@ exn_trace(JSTracer *trc, JSObject *obj);
 static void
 exn_finalize(JSContext *cx, JSObject *obj);
 
-static JSBool
-exn_enumerate(JSContext *cx, JSObject *obj);
-
 static JSBool
 exn_resolve(JSContext *cx, JSObject *obj, jsid id, uintN flags,
             JSObject **objp);
 
 Class js_ErrorClass = {
     js_Error_str,
-    JSCLASS_HAS_PRIVATE | JSCLASS_NEW_RESOLVE | JSCLASS_MARK_IS_TRACE |
+    JSCLASS_HAS_PRIVATE | JSCLASS_NEW_RESOLVE |
     JSCLASS_HAS_CACHED_PROTO(JSProto_Error),
     PropertyStub,         /* addProperty */
     PropertyStub,         /* delProperty */
     PropertyStub,         /* getProperty */
     StrictPropertyStub,   /* setProperty */
-    exn_enumerate,
+    EnumerateStub,
     (JSResolveOp)exn_resolve,
     ConvertStub,
     exn_finalize,
@@ -105,7 +103,7 @@ Class js_ErrorClass = {
     NULL,                 /* construct   */
     NULL,                 /* xdrObject   */
     NULL,                 /* hasInstance */
-    JS_CLASS_TRACE(exn_trace)
+    exn_trace
 };
 
 typedef struct JSStackTraceElem {
@@ -182,7 +180,7 @@ CopyErrorReport(JSContext *cx, JSErrorReport *report)
      */
     mallocSize = sizeof(JSErrorReport) + argsArraySize + argsCopySize +
                  ucmessageSize + uclinebufSize + linebufSize + filenameSize;
-    cursor = (uint8 *)cx->malloc(mallocSize);
+    cursor = (uint8 *)cx->malloc_(mallocSize);
     if (!cursor)
         return NULL;
 
@@ -269,7 +267,7 @@ InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
     JSErrorReporter older;
     JSExceptionState *state;
     jsid callerid;
-    JSStackFrame *fp, *fpstop;
+    StackFrame *fp, *fpstop;
     size_t stackDepth, valueCount, size;
     JSBool overflow;
     JSExnPrivate *priv;
@@ -296,9 +294,9 @@ InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
     stackDepth = 0;
     valueCount = 0;
     for (fp = js_GetTopStackFrame(cx); fp; fp = fp->prev()) {
-        if (fp->scopeChain().compartment() != cx->compartment)
+        if (fp->compartment() != cx->compartment)
             break;
-        if (fp->isFunctionFrame() && !fp->isEvalFrame()) {
+        if (fp->isNonEvalFunctionFrame()) {
             Value v = NullValue();
             if (checkAccess &&
                 !checkAccess(cx, &fp->callee(), callerid, JSACC_READ, &v)) {
@@ -321,7 +319,7 @@ InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
         js_ReportAllocationOverflow(cx);
         return JS_FALSE;
     }
-    priv = (JSExnPrivate *)cx->malloc(size);
+    priv = (JSExnPrivate *)cx->malloc_(size);
     if (!priv)
         return JS_FALSE;
 
@@ -339,14 +337,14 @@ InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
     values = GetStackTraceValueBuffer(priv);
     elem = priv->stackElems;
     for (fp = js_GetTopStackFrame(cx); fp != fpstop; fp = fp->prev()) {
-        if (fp->scopeChain().compartment() != cx->compartment)
+        if (fp->compartment() != cx->compartment)
             break;
         if (!fp->isFunctionFrame() || fp->isEvalFrame()) {
             elem->funName = NULL;
             elem->argc = 0;
         } else {
             elem->funName = fp->fun()->atom
-                            ? ATOM_TO_STRING(fp->fun()->atom)
+                            ? fp->fun()->atom
                             : cx->runtime->emptyString;
             elem->argc = fp->numActualArgs();
             fp->forEachCanonicalActualArg(CopyTo(Valueify(values)));
@@ -428,37 +426,11 @@ exn_finalize(JSContext *cx, JSObject *obj)
     priv = GetExnPrivate(cx, obj);
     if (priv) {
         if (priv->errorReport)
-            cx->free(priv->errorReport);
-        cx->free(priv);
+            cx->free_(priv->errorReport);
+        cx->free_(priv);
     }
 }
 
-static JSBool
-exn_enumerate(JSContext *cx, JSObject *obj)
-{
-    JSAtomState *atomState;
-    uintN i;
-    JSAtom *atom;
-    JSObject *pobj;
-    JSProperty *prop;
-
-    JS_STATIC_ASSERT(sizeof(JSAtomState) <= (size_t)(uint16)-1);
-    static const uint16 offsets[] = {
-        (uint16)offsetof(JSAtomState, messageAtom),
-        (uint16)offsetof(JSAtomState, fileNameAtom),
-        (uint16)offsetof(JSAtomState, lineNumberAtom),
-        (uint16)offsetof(JSAtomState, stackAtom),
-    };
-
-    atomState = &cx->runtime->atomState;
-    for (i = 0; i != JS_ARRAY_LENGTH(offsets); ++i) {
-        atom = *(JSAtom **)((uint8 *)atomState + offsets[i]);
-        if (!js_LookupProperty(cx, obj, ATOM_TO_JSID(atom), &pobj, &prop))
-            return JS_FALSE;
-    }
-    return JS_TRUE;
-}
-
 static JSBool
 exn_resolve(JSContext *cx, JSObject *obj, jsid id, uintN flags,
             JSObject **objp)
@@ -469,6 +441,7 @@ exn_resolve(JSContext *cx, JSObject *obj, jsid id, uintN flags,
     JSString *stack;
     const char *prop;
     jsval v;
+    uintN attrs;
 
     *objp = NULL;
     priv = GetExnPrivate(cx, obj);
@@ -476,46 +449,59 @@ exn_resolve(JSContext *cx, JSObject *obj, jsid id, uintN flags,
         str = JSID_TO_STRING(id);
 
         atom = cx->runtime->atomState.messageAtom;
-        if (str == ATOM_TO_STRING(atom)) {
+        if (str == atom) {
             prop = js_message_str;
+
+            /*
+             * Per ES5 15.11.1.1, if Error is called with no argument or with
+             * undefined as the argument, it returns an Error object with no
+             * own message property.
+             */
+            if (!priv->message)
+                return true;
+
             v = STRING_TO_JSVAL(priv->message);
+            attrs = 0;
             goto define;
         }
 
         atom = cx->runtime->atomState.fileNameAtom;
-        if (str == ATOM_TO_STRING(atom)) {
+        if (str == atom) {
             prop = js_fileName_str;
             v = STRING_TO_JSVAL(priv->filename);
+            attrs = JSPROP_ENUMERATE;
             goto define;
         }
 
         atom = cx->runtime->atomState.lineNumberAtom;
-        if (str == ATOM_TO_STRING(atom)) {
+        if (str == atom) {
             prop = js_lineNumber_str;
             v = INT_TO_JSVAL(priv->lineno);
+            attrs = JSPROP_ENUMERATE;
             goto define;
         }
 
         atom = cx->runtime->atomState.stackAtom;
-        if (str == ATOM_TO_STRING(atom)) {
+        if (str == atom) {
             stack = StackTraceToString(cx, priv);
             if (!stack)
-                return JS_FALSE;
+                return false;
 
             /* Allow to GC all things that were used to build stack trace. */
             priv->stackDepth = 0;
             prop = js_stack_str;
             v = STRING_TO_JSVAL(stack);
+            attrs = JSPROP_ENUMERATE;
             goto define;
         }
     }
-    return JS_TRUE;
+    return true;
 
   define:
-    if (!JS_DefineProperty(cx, obj, prop, v, NULL, NULL, JSPROP_ENUMERATE))
-        return JS_FALSE;
+    if (!JS_DefineProperty(cx, obj, prop, v, NULL, NULL, attrs))
+        return false;
     *objp = obj;
-    return JS_TRUE;
+    return true;
 }
 
 JSErrorReport *
@@ -605,7 +591,7 @@ StackTraceToString(JSContext *cx, JSExnPrivate *priv)
             if (stackmax >= STACK_LENGTH_LIMIT)                               \
                 goto done;                                                    \
             stackmax = stackmax ? 2 * stackmax : 64;                          \
-            ptr_ = cx->realloc(stackbuf, (stackmax+1) * sizeof(jschar));      \
+            ptr_ = cx->realloc_(stackbuf, (stackmax+1) * sizeof(jschar));      \
             if (!ptr_)                                                        \
                 goto bad;                                                     \
             stackbuf = (jschar *) ptr_;                                       \
@@ -628,7 +614,7 @@ StackTraceToString(JSContext *cx, JSExnPrivate *priv)
                 goto done;                                                    \
             }                                                                 \
             stackmax = JS_BIT(JS_CeilingLog2(stacklen + length_));            \
-            ptr_ = cx->realloc(stackbuf, (stackmax+1) * sizeof(jschar));      \
+            ptr_ = cx->realloc_(stackbuf, (stackmax+1) * sizeof(jschar));      \
             if (!ptr_)                                                        \
                 goto bad;                                                     \
             stackbuf = (jschar *) ptr_;                                       \
@@ -679,7 +665,7 @@ StackTraceToString(JSContext *cx, JSExnPrivate *priv)
          * don't use JS_realloc here; simply let the oversized allocation
          * be owned by the string in that rare case.
          */
-        void *shrunk = cx->realloc(stackbuf, (stacklen+1) * sizeof(jschar));
+        void *shrunk = cx->realloc_(stackbuf, (stacklen+1) * sizeof(jschar));
         if (shrunk)
             stackbuf = (jschar *) shrunk;
     }
@@ -691,7 +677,7 @@ StackTraceToString(JSContext *cx, JSExnPrivate *priv)
 
   bad:
     if (stackbuf)
-        cx->free(stackbuf);
+        cx->free_(stackbuf);
     return NULL;
 }
 
@@ -707,7 +693,7 @@ static JSBool
 Exception(JSContext *cx, uintN argc, Value *vp)
 {
     JSString *message, *filename;
-    JSStackFrame *fp;
+    StackFrame *fp;
 
     /*
      * ECMA ed. 3, 15.11.1 requires Error, etc., to construct even when
@@ -740,13 +726,13 @@ Exception(JSContext *cx, uintN argc, Value *vp)
 
     /* Set the 'message' property. */
     Value *argv = vp + 2;
-    if (argc != 0) {
+    if (argc != 0 && !argv[0].isUndefined()) {
         message = js_ValueToString(cx, argv[0]);
         if (!message)
             return JS_FALSE;
         argv[0].setString(message);
     } else {
-        message = cx->runtime->emptyString;
+        message = NULL;
     }
 
     /* Set the 'fileName' property. */
@@ -819,7 +805,7 @@ exn_toString(JSContext *cx, uintN argc, Value *vp)
         name_length = name->length();
         message_length = message->length();
         length = (name_length ? name_length + 2 : 0) + message_length;
-        cp = chars = (jschar *) cx->malloc((length + 1) * sizeof(jschar));
+        cp = chars = (jschar *) cx->malloc_((length + 1) * sizeof(jschar));
         if (!chars)
             return JS_FALSE;
 
@@ -840,7 +826,7 @@ exn_toString(JSContext *cx, uintN argc, Value *vp)
 
         result = js_NewString(cx, chars, length);
         if (!result) {
-            cx->free(chars);
+            cx->free_(chars);
             return JS_FALSE;
         }
     } else {
@@ -930,7 +916,7 @@ exn_toSource(JSContext *cx, uintN argc, Value *vp)
             }
         }
 
-        cp = chars = (jschar *) cx->malloc((length + 1) * sizeof(jschar));
+        cp = chars = (jschar *) cx->malloc_((length + 1) * sizeof(jschar));
         if (!chars)
             return false;
 
@@ -980,7 +966,7 @@ exn_toSource(JSContext *cx, uintN argc, Value *vp)
 
         result = js_NewString(cx, chars, length);
         if (!result) {
-            cx->free(chars);
+            cx->free_(chars);
             return false;
         }
         vp->setString(result);
@@ -1057,18 +1043,14 @@ js_InitExceptionClasses(JSContext *cx, JSObject *obj)
 
         /* Add properties to the prototype. */
         JSAutoResolveFlags rf(cx, JSRESOLVE_QUALIFIED | JSRESOLVE_DECLARING);
-        if (!js_DefineNativeProperty(cx, proto, nameId, StringValue(atom),
-                                     PropertyStub, StrictPropertyStub, 
-                                     JSPROP_ENUMERATE, 0, 0, NULL) ||
-            !js_DefineNativeProperty(cx, proto, messageId, empty,
-                                     PropertyStub, StrictPropertyStub,
-                                     JSPROP_ENUMERATE, 0, 0, NULL) ||
-            !js_DefineNativeProperty(cx, proto, fileNameId, empty,
-                                     PropertyStub, StrictPropertyStub,
-                                     JSPROP_ENUMERATE, 0, 0, NULL) ||
-            !js_DefineNativeProperty(cx, proto, lineNumberId, Valueify(JSVAL_ZERO),
-                                     PropertyStub, StrictPropertyStub,
-                                     JSPROP_ENUMERATE, 0, 0, NULL)) {
+        if (!DefineNativeProperty(cx, proto, nameId, StringValue(atom),
+                                  PropertyStub, StrictPropertyStub, 0, 0, 0) ||
+            !DefineNativeProperty(cx, proto, messageId, empty,
+                                  PropertyStub, StrictPropertyStub, 0, 0, 0) ||
+            !DefineNativeProperty(cx, proto, fileNameId, empty,
+                                  PropertyStub, StrictPropertyStub, JSPROP_ENUMERATE, 0, 0) ||
+            !DefineNativeProperty(cx, proto, lineNumberId, Valueify(JSVAL_ZERO),
+                                  PropertyStub, StrictPropertyStub, JSPROP_ENUMERATE, 0, 0)) {
             return NULL;
         }
     }
@@ -1280,9 +1262,10 @@ js_ReportUncaughtException(JSContext *cx)
         report.filename = filename.ptr();
         report.lineno = (uintN) lineno;
         if (JSVAL_IS_STRING(roots[2])) {
-            report.ucmessage = js_GetStringChars(cx, JSVAL_TO_STRING(roots[2]));
-            if (!report.ucmessage)
+            JSFixedString *fixed = JSVAL_TO_STRING(roots[2])->ensureFixed(cx);
+            if (!fixed)
                 return false;
+            report.ucmessage = fixed->chars();
         }
     }
 
