@@ -17,10 +17,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Franky Braem
+ * Contributor(s): Igor Bukanov
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -37,36 +37,25 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-var summary = "Do not crash when qn->uri is null";
-var BUGNUMBER = 323338;
-var actual = 'No Crash';
-var expect = 'No Crash';
+var summary = 'GetXMLEntity should not assume FastAppendChar is infallible';
+var BUGNUMBER = 458679;
+var actual = '';
+var expect = '';
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
-testFunc();
-testFunc();
-
-function testFunc()
+try
 {
-  var htmlXML =
-   <html>
-    <body>
-     <div>
-      <div id="summary" />
-      <div id="desc" />
-     </div>
-    </body>
-   </html>;
-  var childs = htmlXML.children();
+    var x = "<";
 
-  var el = htmlXML.body.div..div.(function::attribute('id') == 'summary');
-  el.div += <div>
-              <strong>Prototype:</strong>
-              Test
-              <br />
-             </div>;
+    while (x.length < 12000000)
+        x += x;
+
+    <e4x>{x}</e4x>;
+}
+catch(ex)
+{
 }
 
 TEST(1, expect, actual);
