@@ -41,50 +41,8 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-START("13.5.2 - XMLList Constructor");
+START("13.5.4.15 - XMLList normalize()");
 
-x = new XMLList();
-TEST(1, "xml", typeof(x));  
-TEST(2, true, x instanceof XMLList);
-
-// Load from another XMLList
-// Make sure it is copied if it's an XMLList
-x =
-<>
-    <alpha>one</alpha>
-    <bravo>two</bravo>
-</>;
-
-y = new XMLList(x);
-
-x += <charlie>three</charlie>;
-
-TEST(3, "<alpha>one</alpha>\n<bravo>two</bravo>", y.toString());
-  
-// Load from one XML type
-x = new XMLList(<alpha>one</alpha>);
-TEST_XML(4, "<alpha>one</alpha>", x);
-
-// Load from Anonymous
-x = new XMLList(<><alpha>one</alpha><bravo>two</bravo></>);
-TEST(5, "<alpha>one</alpha>\n<bravo>two</bravo>", x.toString());
-
-// Load from Anonymous as string
-x = new XMLList(<><alpha>one</alpha><bravo>two</bravo></>);
-TEST(6, "<alpha>one</alpha>\n<bravo>two</bravo>", x.toString());
-
-// Load from single textnode
-x = new XMLList("foobar");
-TEST_XML(7, "foobar", x);
-
-x = XMLList(7);
-TEST_XML(8, 7, x);
-
-// Undefined and null should behave like ""
-x = new XMLList(null);
-TEST_XML(9, "", x);
-
-x = new XMLList(undefined);
-TEST_XML(10, "", x);
+TEST(1, true, XMLList.prototype.hasOwnProperty("normalize"));
 
 END();
