@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,8 +36,8 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 355829;
-var summary = 'js_ValueToObject should return the original object if OBJ_DEFAULT_VALUE returns a primitive value';
+var BUGNUMBER = 470187;
+var summary = 'Do not assert: entry->kpc == (jsbytecode*) atoms[index]';
 var actual = '';
 var expect = '';
 
@@ -51,21 +51,8 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
-
-  expect = actual = 'Good conversion';
-
-  var primitiveValues = [
-    true, false, 0, 1, -2, 0.1, -2e100, 0/0, 1/0, -1/1,  "", "xxx",
-    undefined, null
-    ];
-
-  for (var i = 0; i != primitiveValues.length; ++i) {
-    var v = primitiveValues[i];
-    var obj = { valueOf: function() { return v; } };
-    var obj2 = Object(obj);
-    if (obj !== obj2)
-      actual = "Bad conversion for '"+v + "'";
-  }
+ 
+  for (var j=0;j<3;++j) ({valueOf: function(){return 2}}) - /x/;
 
   reportCompare(expect, actual, summary);
 
