@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2007
+ * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Igor Bukanov
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,19 +35,44 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 358594;
-var summary = 'Do not crash on uneval(this).';
+var BUGNUMBER = 354499;
+var summary = 'Iterating over Array elements';
 var actual = '';
 var expect = '';
 
-printBugNumber(BUGNUMBER);
-printStatus (summary);
 
-// don't crash|assert
-function f() { } 
-f.__proto__ = this; 
-Object.defineProperty(this, "m", { set: f, enumerable: true, configurable: true });
-uneval(this);
-reportCompare(expect, actual, summary);
+//-----------------------------------------------------------------------------
+test();
+//-----------------------------------------------------------------------------
+
+function test()
+{
+  enterFunc ('test');
+  printBugNumber(BUGNUMBER);
+  printStatus (summary);
+
+  expect = actual = 'No Crash';
+
+  var obj = {get a(){ return new Object(); }};
+
+  function setter(v)
+  {
+    // Push out obj.a from all temp roots
+    var tmp = { get toString() { return new Object(); }};
+    try { String(tmp); } catch (e) {  }
+    gc();
+  }
+
+  Array.prototype.__defineGetter__(0, function() { });
+  Array.prototype.__defineSetter__(0, setter);
+
+  for (var i in Iterator(obj))
+    print(uneval(i));
+
+  delete Array.prototype[0];
+
+  reportCompare(expect, actual, summary);
+
+  exitFunc ('test');
+}
