@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2007
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Igor Bukanov
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,10 +36,10 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 465460;
-var summary = 'TM: valueOf in a loop: do not assert';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 385133;
+var summary = 'Do not crash due to recursion with watch, setter, delete, generator';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 
 //-----------------------------------------------------------------------------
@@ -51,13 +51,17 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  jit(true);
-
-  for (let d=0;d<2;++d) for (let a=0;a<1;++a) "" + (d && function(){});
-
-  jit(false);
 
+  try
+  { 
+    Object.defineProperty(this, "x", { set: {}.watch, enumerable: true, configurable: true });
+    function g() { x = 1; yield; }
+    g().next();
+  }
+  catch(ex)
+  {
+    print(ex + '');
+  }
   reportCompare(expect, actual, summary);
 
   exitFunc ('test');
