@@ -46,6 +46,9 @@
 #include "nsSVGFilterFrame.h"
 #include "nsSVGClipPathFrame.h"
 #include "nsSVGMaskFrame.h"
+#include "gfxPlatform.h"
+#include "gfxDrawable.h"
+#include "nsSVGPaintServerFrame.h"
 
 // ----------------------------------------------------------------------
 
@@ -57,9 +60,8 @@ nsSVGIntegrationUtils::UsingEffectsForFrame(const nsIFrame* aFrame)
           !aFrame->IsFrameOfType(nsIFrame::eSVG);
 }
 
-// Get the union the frame border-box rects over all continuations,
-// relative to aFirst. This defines "user space" for non-SVG frames.
-static nsRect GetNonSVGUserSpace(nsIFrame* aFirst)
+/* static */ nsRect
+nsSVGIntegrationUtils::GetNonSVGUserSpace(nsIFrame* aFirst)
 {
   NS_ASSERTION(!aFirst->GetPrevContinuation(), "Not first continuation");
   return nsLayoutUtils::GetAllInFlowRectsUnion(aFirst, aFirst);
@@ -68,10 +70,11 @@ static nsRect GetNonSVGUserSpace(nsIFrame* aFirst)
 static nsRect
 GetPreEffectsOverflowRect(nsIFrame* aFrame)
 {
-  nsRect* r = static_cast<nsRect*>(aFrame->GetProperty(nsGkAtoms::preEffectsBBoxProperty));
+  nsRect* r = static_cast<nsRect*>
+    (aFrame->Properties().Get(nsIFrame::PreEffectsBBoxProperty()));
   if (r)
     return *r;
-  return aFrame->GetOverflowRect();
+  return aFrame->GetVisualOverflowRect();
 }
 
 struct BBoxCollector : public nsLayoutUtils::BoxCallback {
@@ -144,12 +147,18 @@ nsSVGIntegrationUtils::GetInvalidAreaForChangedSource(nsIFrame* aFrame,
     nsSVGEffects::GetEffectProperties(firstFrame);
   if (!effectProperties.mFilter)
     return aInvalidRect;
-  nsSVGFilterFrame* filterFrame = nsSVGEffects::GetFilterFrame(firstFrame);
+
+  nsSVGFilterProperty *prop = nsSVGEffects::GetFilterProperty(firstFrame);
+  if (!prop || !prop->IsInObserverList()) {
+    return aInvalidRect;
+  }
+
+  nsSVGFilterFrame* filterFrame = prop->GetFilterFrame();
   if (!filterFrame) {
     // The frame is either not there or not currently available,
     // perhaps because we're in the middle of tearing stuff down.
     // Be conservative.
-    return aFrame->GetOverflowRect();
+    return aFrame->GetVisualOverflowRect();
   }
 
   PRInt32 appUnitsPerDevPixel = aFrame->PresContext()->AppUnitsPerDevPixel();
@@ -201,25 +210,28 @@ class RegularFramePaintCallback : public nsSVGFilterPaintCallback
 public:
   RegularFramePaintCallback(nsDisplayListBuilder* aBuilder,
                             nsDisplayList* aInnerList,
+                            nsIFrame* aFrame,
                             const nsPoint& aOffset)
-    : mBuilder(aBuilder), mInnerList(aInnerList), mOffset(aOffset) {}
+    : mBuilder(aBuilder), mInnerList(aInnerList), mFrame(aFrame),
+      mOffset(aOffset) {}
 
   virtual void Paint(nsSVGRenderState *aContext, nsIFrame *aTarget,
                      const nsIntRect* aDirtyRect)
   {
-    nsIRenderingContext* ctx = aContext->GetRenderingContext(aTarget);
-    nsIRenderingContext::AutoPushTranslation push(ctx, -mOffset.x, -mOffset.y);
-    mInnerList->Paint(mBuilder, ctx);
+    nsRenderingContext* ctx = aContext->GetRenderingContext(aTarget);
+    nsRenderingContext::AutoPushTranslation push(ctx, -mOffset);
+    mInnerList->PaintForFrame(mBuilder, ctx, mFrame, nsDisplayList::PAINT_DEFAULT);
   }
 
 private:
   nsDisplayListBuilder* mBuilder;
   nsDisplayList* mInnerList;
+  nsIFrame* mFrame;
   nsPoint mOffset;
 };
 
 void
-nsSVGIntegrationUtils::PaintFramesWithEffects(nsIRenderingContext* aCtx,
+nsSVGIntegrationUtils::PaintFramesWithEffects(nsRenderingContext* aCtx,
                                               nsIFrame* aEffectsFrame,
                                               const nsRect& aDirtyRect,
                                               nsDisplayListBuilder* aBuilder,
@@ -274,7 +286,7 @@ nsSVGIntegrationUtils::PaintFramesWithEffects(nsIRenderingContext* aCtx,
   nsRect userSpaceRect = GetNonSVGUserSpace(firstFrame) + aBuilder->ToReferenceFrame(firstFrame);
   PRInt32 appUnitsPerDevPixel = aEffectsFrame->PresContext()->AppUnitsPerDevPixel();
   userSpaceRect = userSpaceRect.ToNearestPixels(appUnitsPerDevPixel).ToAppUnits(appUnitsPerDevPixel);
-  aCtx->Translate(userSpaceRect.x, userSpaceRect.y);
+  aCtx->Translate(userSpaceRect.TopLeft());
 
   gfxMatrix matrix = GetInitialMatrix(aEffectsFrame);
 
@@ -284,6 +296,7 @@ nsSVGIntegrationUtils::PaintFramesWithEffects(nsIRenderingContext* aCtx,
   if (opacity != 1.0f || maskFrame || (clipPathFrame && !isTrivialClip)) {
     complexEffects = PR_TRUE;
     gfx->Save();
+    aCtx->IntersectClip(aEffectsFrame->GetVisualOverflowRect());
     gfx->PushGroup(gfxASurface::CONTENT_COLOR_ALPHA);
   }
 
@@ -297,13 +310,15 @@ nsSVGIntegrationUtils::PaintFramesWithEffects(nsIRenderingContext* aCtx,
 
   /* Paint the child */
   if (filterFrame) {
-    RegularFramePaintCallback paint(aBuilder, aInnerList, userSpaceRect.TopLeft());
+    RegularFramePaintCallback paint(aBuilder, aInnerList, aEffectsFrame,
+                                    userSpaceRect.TopLeft());
     nsIntRect r = (aDirtyRect - userSpaceRect.TopLeft()).ToOutsidePixels(appUnitsPerDevPixel);
     filterFrame->FilterPaint(&svgContext, aEffectsFrame, &paint, &r);
   } else {
     gfx->SetMatrix(savedCTM);
-    aInnerList->Paint(aBuilder, aCtx);
-    aCtx->Translate(userSpaceRect.x, userSpaceRect.y);
+    aInnerList->PaintForFrame(aBuilder, aCtx, aEffectsFrame,
+                              nsDisplayList::PAINT_DEFAULT);
+    aCtx->Translate(userSpaceRect.TopLeft());
   }
 
   if (clipPathFrame && isTrivialClip) {
@@ -392,3 +407,158 @@ nsSVGIntegrationUtils::GetSVGBBoxForNonSVGFrame(nsIFrame* aNonSVGFrame)
   result.ScaleInverse(presContext->AppUnitsPerCSSPixel());
   return result;
 }
+
+class PaintFrameCallback : public gfxDrawingCallback {
+public:
+  PaintFrameCallback(nsIFrame* aFrame,
+                     nsIFrame* aTarget,
+                     const nsSize aPaintServerSize,
+                     const gfxIntSize aRenderSize)
+   : mFrame(aFrame)
+   , mTarget(aTarget)
+   , mPaintServerSize(aPaintServerSize)
+   , mRenderSize(aRenderSize)
+  {}
+  virtual PRBool operator()(gfxContext* aContext,
+                            const gfxRect& aFillRect,
+                            const gfxPattern::GraphicsFilter& aFilter,
+                            const gfxMatrix& aTransform);
+private:
+  nsIFrame* mFrame;
+  nsIFrame* mTarget;
+  nsSize mPaintServerSize;
+  gfxIntSize mRenderSize;
+};
+
+PRBool
+PaintFrameCallback::operator()(gfxContext* aContext,
+                               const gfxRect& aFillRect,
+                               const gfxPattern::GraphicsFilter& aFilter,
+                               const gfxMatrix& aTransform)
+{
+  if (mFrame->GetStateBits() & NS_FRAME_DRAWING_AS_PAINTSERVER)
+    return PR_FALSE;
+
+  mFrame->AddStateBits(NS_FRAME_DRAWING_AS_PAINTSERVER);
+
+  nsSVGRenderState renderState(aContext);
+  aContext->Save();
+
+  // Clip to aFillRect so that we don't paint outside.
+  aContext->NewPath();
+  aContext->Rectangle(aFillRect);
+  aContext->Clip();
+  gfxMatrix savedMatrix(aContext->CurrentMatrix());
+
+  aContext->Multiply(gfxMatrix(aTransform).Invert());
+
+  // nsLayoutUtils::PaintFrame will anchor its painting at mFrame. But we want
+  // to have it anchored at the top left corner of the bounding box of all of
+  // mFrame's continuations. So we add a translation transform.
+  nsRect bbox = nsSVGIntegrationUtils::GetNonSVGUserSpace(mFrame);
+  PRInt32 appUnitsPerDevPixel = mFrame->PresContext()->AppUnitsPerDevPixel();
+  gfxPoint offset = gfxPoint(bbox.x, bbox.y) / appUnitsPerDevPixel;
+  aContext->Multiply(gfxMatrix().Translate(-offset));
+
+  gfxSize paintServerSize =
+    gfxSize(mPaintServerSize.width, mPaintServerSize.height) /
+      mFrame->PresContext()->AppUnitsPerDevPixel();
+
+  // nsLayoutUtils::PaintFrame wants to render with paintServerSize, but we
+  // want it to render with mRenderSize, so we need to set up a scale transform.
+  gfxFloat scaleX = mRenderSize.width / paintServerSize.width;
+  gfxFloat scaleY = mRenderSize.height / paintServerSize.height;
+  gfxMatrix scaleMatrix = gfxMatrix().Scale(scaleX, scaleY);
+  aContext->Multiply(scaleMatrix);
+
+  // Draw.
+  nsRect dirty(bbox.x, bbox.y, mPaintServerSize.width, mPaintServerSize.height);
+  nsLayoutUtils::PaintFrame(renderState.GetRenderingContext(mTarget), mFrame,
+                            dirty, NS_RGBA(0, 0, 0, 0),
+                            nsLayoutUtils::PAINT_IN_TRANSFORM |
+                            nsLayoutUtils::PAINT_ALL_CONTINUATIONS);
+
+  aContext->SetMatrix(savedMatrix);
+  aContext->Restore();
+
+  mFrame->RemoveStateBits(NS_FRAME_DRAWING_AS_PAINTSERVER);
+
+  return PR_TRUE;
+}
+
+static already_AddRefed<gfxDrawable>
+DrawableFromPaintServer(nsIFrame*         aFrame,
+                        nsIFrame*         aTarget,
+                        const nsSize&     aPaintServerSize,
+                        const gfxIntSize& aRenderSize)
+{
+  // aPaintServerSize is the size that would be filled when using
+  // background-repeat:no-repeat and background-size:auto. For normal background
+  // images, this would be the intrinsic size of the image; for gradients and
+  // patterns this would be the whole target frame fill area.
+  // aRenderSize is what we will be actually filling after accounting for
+  // background-size.
+  if (aFrame->IsFrameOfType(nsIFrame::eSVGPaintServer)) {
+    // aFrame is either a pattern or a gradient. These fill the whole target
+    // frame by default, so aPaintServerSize is the whole target background fill
+    // area.
+    nsSVGPaintServerFrame* server =
+      static_cast<nsSVGPaintServerFrame*>(aFrame);
+
+    gfxRect overrideBounds(0, 0,
+                           aPaintServerSize.width, aPaintServerSize.height);
+    overrideBounds.ScaleInverse(aFrame->PresContext()->AppUnitsPerDevPixel());
+    nsRefPtr<gfxPattern> pattern =
+      server->GetPaintServerPattern(aTarget, 1.0, &overrideBounds);
+
+    if (!pattern)
+      return nsnull;
+
+    // pattern is now set up to fill aPaintServerSize. But we want it to
+    // fill aRenderSize, so we need to add a scaling transform.
+    // We couldn't just have set overrideBounds to aRenderSize - it would have
+    // worked for gradients, but for patterns it would result in a different
+    // pattern size.
+    gfxFloat scaleX = overrideBounds.Width() / aRenderSize.width;
+    gfxFloat scaleY = overrideBounds.Height() / aRenderSize.height;
+    gfxMatrix scaleMatrix = gfxMatrix().Scale(scaleX, scaleY);
+    pattern->SetMatrix(scaleMatrix.Multiply(pattern->GetMatrix()));
+    nsRefPtr<gfxDrawable> drawable =
+      new gfxPatternDrawable(pattern, aRenderSize);
+    return drawable.forget();
+  }
+
+  // We don't want to paint into a surface as long as we don't need to, so we
+  // set up a drawing callback.
+  nsRefPtr<gfxDrawingCallback> cb =
+    new PaintFrameCallback(aFrame, aTarget, aPaintServerSize, aRenderSize);
+  nsRefPtr<gfxDrawable> drawable = new gfxCallbackDrawable(cb, aRenderSize);
+  return drawable.forget();
+}
+
+/* static */ void
+nsSVGIntegrationUtils::DrawPaintServer(nsRenderingContext* aRenderingContext,
+                                       nsIFrame*            aTarget,
+                                       nsIFrame*            aPaintServer,
+                                       gfxPattern::GraphicsFilter aFilter,
+                                       const nsRect&        aDest,
+                                       const nsRect&        aFill,
+                                       const nsPoint&       aAnchor,
+                                       const nsRect&        aDirty,
+                                       const nsSize&        aPaintServerSize)
+{
+  if (aDest.IsEmpty() || aFill.IsEmpty())
+    return;
+
+  PRInt32 appUnitsPerDevPixel = aTarget->PresContext()->AppUnitsPerDevPixel();
+  nsRect destSize = aDest - aDest.TopLeft();
+  nsIntSize roundedOut = destSize.ToOutsidePixels(appUnitsPerDevPixel).Size();
+  gfxIntSize imageSize(roundedOut.width, roundedOut.height);
+  nsRefPtr<gfxDrawable> drawable =
+    DrawableFromPaintServer(aPaintServer, aTarget, aPaintServerSize, imageSize);
+
+  if (drawable) {
+    nsLayoutUtils::DrawPixelSnapped(aRenderingContext, drawable, aFilter,
+                                    aDest, aFill, aAnchor, aDirty);
+  }
+}
