@@ -160,43 +160,49 @@ public:
   NS_IMETHOD GetPath(char **memoryPath)
   {
     if (mType == ChromeUsedRaw) {
-      *memoryPath = strdup("images/chrome/used/raw");
+      *memoryPath = strdup("explicit/images/chrome/used/raw");
     } else if (mType == ChromeUsedUncompressed) {
-      *memoryPath = strdup("images/chrome/used/uncompressed");
+      *memoryPath = strdup("explicit/images/chrome/used/uncompressed");
     } else if (mType == ChromeUnusedRaw) {
-      *memoryPath = strdup("images/chrome/unused/raw");
+      *memoryPath = strdup("explicit/images/chrome/unused/raw");
     } else if (mType == ChromeUnusedUncompressed) {
-      *memoryPath = strdup("images/chrome/unused/uncompressed");
+      *memoryPath = strdup("explicit/images/chrome/unused/uncompressed");
     } else if (mType == ContentUsedRaw) {
-      *memoryPath = strdup("images/content/used/raw");
+      *memoryPath = strdup("explicit/images/content/used/raw");
     } else if (mType == ContentUsedUncompressed) {
-      *memoryPath = strdup("images/content/used/uncompressed");
+      *memoryPath = strdup("explicit/images/content/used/uncompressed");
     } else if (mType == ContentUnusedRaw) {
-      *memoryPath = strdup("images/content/unused/raw");
+      *memoryPath = strdup("explicit/images/content/unused/raw");
     } else if (mType == ContentUnusedUncompressed) {
-      *memoryPath = strdup("images/content/unused/uncompressed");
+      *memoryPath = strdup("explicit/images/content/unused/uncompressed");
     }
     return NS_OK;
   }
 
+  NS_IMETHOD GetKind(PRInt32 *kind)
+  {
+    *kind = MR_HEAP;
+    return NS_OK;
+  }
+
   NS_IMETHOD GetDescription(char **desc)
   {
     if (mType == ChromeUsedRaw) {
-      *desc = strdup("Memory used by in-use chrome images, compressed data");
+      *desc = strdup("Memory used by in-use chrome images (compressed data).");
     } else if (mType == ChromeUsedUncompressed) {
-      *desc = strdup("Memory used by in-use chrome images, uncompressed data");
+      *desc = strdup("Memory used by in-use chrome images (uncompressed data).");
     } else if (mType == ChromeUnusedRaw) {
-      *desc = strdup("Memory used by not in-use chrome images, compressed data");
+      *desc = strdup("Memory used by not in-use chrome images (compressed data).");
     } else if (mType == ChromeUnusedUncompressed) {
-      *desc = strdup("Memory used by not in-use chrome images, uncompressed data");
+      *desc = strdup("Memory used by not in-use chrome images (uncompressed data).");
     } else if (mType == ContentUsedRaw) {
-      *desc = strdup("Memory used by in-use content images, compressed data");
+      *desc = strdup("Memory used by in-use content images (compressed data).");
     } else if (mType == ContentUsedUncompressed) {
-      *desc = strdup("Memory used by in-use content images, uncompressed data");
+      *desc = strdup("Memory used by in-use content images (uncompressed data).");
     } else if (mType == ContentUnusedRaw) {
-      *desc = strdup("Memory used by not in-use content images, compressed data");
+      *desc = strdup("Memory used by not in-use content images (compressed data).");
     } else if (mType == ContentUnusedUncompressed) {
-      *desc = strdup("Memory used by not in-use content images, uncompressed data");
+      *desc = strdup("Memory used by not in-use content images (uncompressed data).");
     }
     return NS_OK;
   }
@@ -1171,6 +1177,9 @@ PRBool imgLoader::ValidateRequestWithNewChannel(imgRequest *request,
     rv = CreateNewProxyForRequest(request, aLoadGroup, aObserver,
                                   aLoadFlags, aExistingRequest, 
                                   reinterpret_cast<imgIRequest **>(aProxyRequest));
+    if (NS_FAILED(rv)) {
+      return PR_FALSE;
+    }
 
     if (*aProxyRequest) {
       imgRequestProxy* proxy = static_cast<imgRequestProxy*>(*aProxyRequest);
@@ -1692,6 +1701,10 @@ NS_IMETHODIMP imgLoader::LoadImage(nsIURI *aURI,
     LOG_MSG(gImgLog, "imgLoader::LoadImage", "creating proxy request.");
     rv = CreateNewProxyForRequest(request, aLoadGroup, aObserver,
                                   requestFlags, aRequest, _retval);
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
+
     imgRequestProxy *proxy = static_cast<imgRequestProxy *>(*_retval);
 
     // Make sure that OnStatus/OnProgress calls have the right request set, if
