@@ -46,16 +46,22 @@
 #include "nsHtml5TreeBuilder.h"
 #include "nsHtml5UTF16Buffer.h"
 #include "nsHtml5Module.h"
+#include "nsIObserverService.h"
+#include "nsIServiceManager.h"
+#include "mozilla/Services.h"
 
 // static
 PRBool nsHtml5Module::sEnabled = PR_FALSE;
+PRBool nsHtml5Module::sOffMainThread = PR_TRUE;
+nsIThread* nsHtml5Module::sStreamParserThread = nsnull;
+nsIThread* nsHtml5Module::sMainThread = nsnull;
 
 // static
 void
 nsHtml5Module::InitializeStatics()
 {
-#if 0
-  nsContentUtils::AddBoolPrefVarCache("html5.enable", &sEnabled);
+  nsContentUtils::AddBoolPrefVarCache("html5.parser.enable", &sEnabled);
+  nsContentUtils::AddBoolPrefVarCache("html5.offmainthread", &sOffMainThread);
   nsHtml5Atoms::AddRefAtoms();
   nsHtml5AttributeName::initializeStatics();
   nsHtml5ElementName::initializeStatics();
@@ -66,17 +72,16 @@ nsHtml5Module::InitializeStatics()
   nsHtml5Tokenizer::initializeStatics();
   nsHtml5TreeBuilder::initializeStatics();
   nsHtml5UTF16Buffer::initializeStatics();
+  nsHtml5StreamParser::InitializeStatics();
 #ifdef DEBUG
   sNsHtml5ModuleInitialized = PR_TRUE;
 #endif
-#endif
 }
 
 // static
 void
 nsHtml5Module::ReleaseStatics()
 {
-#if 0
 #ifdef DEBUG
   sNsHtml5ModuleInitialized = PR_FALSE;
 #endif
@@ -89,7 +94,8 @@ nsHtml5Module::ReleaseStatics()
   nsHtml5Tokenizer::releaseStatics();
   nsHtml5TreeBuilder::releaseStatics();
   nsHtml5UTF16Buffer::releaseStatics();
-#endif
+  NS_IF_RELEASE(sStreamParserThread);
+  NS_IF_RELEASE(sMainThread);
 }
 
 // static
@@ -111,6 +117,52 @@ nsHtml5Module::Initialize(nsIParser* aParser, nsIDocument* aDoc, nsIURI* aURI, n
   return parser->Initialize(aDoc, aURI, aContainer, aChannel);
 }
 
+class nsHtml5ParserThreadTerminator : public nsIObserver
+{
+  public:
+    NS_DECL_ISUPPORTS
+    nsHtml5ParserThreadTerminator(nsIThread* aThread)
+      : mThread(aThread)
+    {}
+    NS_IMETHODIMP Observe(nsISupports *, const char *topic, const PRUnichar *)
+    {
+      NS_ASSERTION(!strcmp(topic, "xpcom-shutdown-threads"), 
+                   "Unexpected topic");
+      if (mThread) {
+        mThread->Shutdown();
+        mThread = nsnull;
+      }
+      return NS_OK;
+    }
+  private:
+    nsCOMPtr<nsIThread> mThread;
+};
+
+NS_IMPL_ISUPPORTS1(nsHtml5ParserThreadTerminator, nsIObserver)
+
+// static 
+nsIThread*
+nsHtml5Module::GetStreamParserThread()
+{
+  if (sOffMainThread) {
+    if (!sStreamParserThread) {
+      NS_NewThread(&sStreamParserThread);
+      NS_ASSERTION(sStreamParserThread, "Thread creation failed!");
+      nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
+      NS_ASSERTION(os, "do_GetService failed");
+      os->AddObserver(new nsHtml5ParserThreadTerminator(sStreamParserThread), 
+                      "xpcom-shutdown-threads",
+                      PR_FALSE);
+    }
+    return sStreamParserThread;
+  }
+  if (!sMainThread) {
+    NS_GetMainThread(&sMainThread);
+    NS_ASSERTION(sMainThread, "Main thread getter failed");
+  }
+  return sMainThread;
+}
+
 #ifdef DEBUG
 PRBool nsHtml5Module::sNsHtml5ModuleInitialized = PR_FALSE;
 #endif
