@@ -162,6 +162,7 @@ static PRBool disableLocking  = PR_FALSE;
 static PRBool ignoreErrors    = PR_FALSE;
 static PRBool enableSessionTickets = PR_FALSE;
 static PRBool enableCompression    = PR_FALSE;
+static PRBool enableFalseStart     = PR_FALSE;
 
 PRIntervalTime maxInterval    = PR_INTERVAL_NO_TIMEOUT;
 
@@ -197,7 +198,8 @@ Usage(const char *progName)
         "       -U means enable throttling up threads\n"
 	"       -B bypasses the PKCS11 layer for SSL encryption and MACing\n"
 	"       -u enable TLS Session Ticket extension\n"
-	"       -z enable compression\n",
+	"       -z enable compression\n"
+	"       -g enable false start\n",
 	progName);
     exit(1);
 }
@@ -266,7 +268,7 @@ mySSLAuthCertificate(void *arg, PRFileDesc *fd, PRBool checkSig,
     /* invoke the "default" AuthCert handler. */
     rv = SSL_AuthCertificate(arg, fd, checkSig, isServer);
 
-    PR_AtomicIncrement(&certsTested);
+    PR_ATOMIC_INCREMENT(&certsTested);
     if (rv == SECSuccess) {
 	fputs("strsclnt: -- SSL: Server Certificate Validated.\n", stderr);
     }
@@ -358,7 +360,7 @@ printSecurityInfo(PRFileDesc *fd)
 
 #define MAX_THREADS 128
 
-typedef int startFn(void *a, void *b, int c);
+typedef int startFn(void *a, void *b, int c, int d);
 
 
 static PRInt32     numConnected;
@@ -372,6 +374,7 @@ typedef struct perThreadStr {
     startFn  *  startFunc;
     PRThread *  prThread;
     PRBool	inUse;
+    PRInt32     socketDomain;
 } perThread;
 
 perThread threads[MAX_THREADS];
@@ -427,7 +430,8 @@ thread_wrapper(void * arg)
         }
         PR_Unlock(threadLock);
         if (doop) {
-            slot->rv = (* slot->startFunc)(slot->a, slot->b, slot->tid);
+            slot->rv = (* slot->startFunc)(slot->a, slot->b, slot->tid,
+                                           slot->socketDomain);
             PRINTF("strsclnt: Thread in slot %d returned %d\n", 
                    slot->tid, slot->rv);
         }
@@ -442,7 +446,8 @@ launch_thread(
     startFn *	startFunc,
     void *	a,
     void *	b,
-    int         tid)
+    int         tid,
+    int         sockDom)
 {
     PRUint32 i;
     perThread * slot;
@@ -460,7 +465,8 @@ launch_thread(
     slot->a = a;
     slot->b = b;
     slot->tid = tid;
-
+    slot->socketDomain = sockDom;
+    
     slot->startFunc = startFunc;
 
     slot->prThread      = PR_CreateThread(PR_USER_THREAD,
@@ -583,7 +589,8 @@ int
 do_writes(
     void *       a,
     void *       b,
-    int          c)
+    int          c,
+    int          d)
 {
     PRFileDesc *	ssl_sock	= (PRFileDesc *)a;
     lockedVars *	lv 		= (lockedVars *)b;
@@ -625,7 +632,7 @@ handle_fdx_connection( PRFileDesc * ssl_sock, int connection)
     lockedVars_AddToCount(&lv, 1);
 
     /* Attempt to launch the writer thread. */
-    result = launch_thread(do_writes, ssl_sock, &lv, connection);
+    result = launch_thread(do_writes, ssl_sock, &lv, connection, -1 /*not used*/);
 
     if (result != SECSuccess) 
     	goto cleanup;
@@ -732,7 +739,7 @@ PRInt32 lastFullHandshakePeerID;
 void
 myHandshakeCallback(PRFileDesc *socket, void *arg) 
 {
-    PR_AtomicSet(&lastFullHandshakePeerID, (PRInt32) arg);
+    PR_ATOMIC_SET(&lastFullHandshakePeerID, (PRInt32) arg);
 }
 
 #endif
@@ -744,7 +751,8 @@ int
 do_connects(
     void *	a,
     void *	b,
-    int         tid)
+    int         tid,
+    PRInt32     socketDomain)
 {
     PRNetAddr  *        addr		= (PRNetAddr *)  a;
     PRFileDesc *        model_sock	= (PRFileDesc *) b;
@@ -758,7 +766,7 @@ do_connects(
 
 retry:
 
-    tcp_sock = PR_OpenTCPSocket(addr->raw.family);
+    tcp_sock = PR_OpenTCPSocket(socketDomain);
     if (tcp_sock == NULL) {
 	errExit("PR_OpenTCPSocket");
     }
@@ -838,7 +846,7 @@ retry:
         static PRInt32 sockPeerID = 0; /* atomically incremented */
         PRInt32 thisPeerID;
 #endif
-        PRInt32 savid = PR_AtomicIncrement(&globalconid);
+        PRInt32 savid = PR_ATOMIC_INCREMENT(&globalconid);
         PRInt32 conid = 1 + (savid - 1) % 100;
         /* don't change peer ID on the very first handshake, which is always
            a full, so the session gets stored into the client cache */
@@ -849,7 +857,7 @@ retry:
 #ifdef USE_SOCK_PEER_ID
         {
             /* force a full handshake by changing the socket peer ID */
-            thisPeerID = PR_AtomicIncrement(&sockPeerID);
+            thisPeerID = PR_ATOMIC_INCREMENT(&sockPeerID);
         } else {
             /* reuse previous sockPeerID for restart handhsake */
             thisPeerID = lastFullHandshakePeerID;
@@ -869,7 +877,7 @@ retry:
 	goto done;
     }
 
-    PR_AtomicIncrement(&numConnected);
+    PR_ATOMIC_INCREMENT(&numConnected);
 
     if (bigBuf.data != NULL) {
 	result = handle_fdx_connection( ssl_sock, tid);
@@ -877,7 +885,7 @@ retry:
 	result = handle_connection( ssl_sock, tid);
     }
 
-    PR_AtomicDecrement(&numConnected);
+    PR_ATOMIC_DECREMENT(&numConnected);
 
 done:
     if (ssl_sock) {
@@ -1086,6 +1094,7 @@ client_main(
     int         rv;
     PRStatus    status;
     PRNetAddr   addr;
+    PRInt32    socketDomain;
 
     status = PR_StringToNetAddr(hostName, &addr);
     if (status == PR_SUCCESS) {
@@ -1113,6 +1122,13 @@ client_main(
 	}
     }
 
+    /* check if SDP is going to be used */
+    if (!PR_GetEnv("NSS_USE_SDP")) {
+        socketDomain = addr.raw.family;
+    } else {
+        socketDomain = PR_AF_INET_SDP;
+    }
+
     /* all suites except RSA_NULL_MD5 are enabled by Domestic Policy */
     NSS_SetDomesticPolicy();
 
@@ -1169,8 +1185,8 @@ client_main(
     }
 
     /* configure model SSL socket. */
-
-    model_sock = PR_OpenTCPSocket(addr.raw.family);
+    
+    model_sock = PR_OpenTCPSocket(socketDomain);
     if (model_sock == NULL) {
 	errExit("PR_OpenTCPSocket for model socket");
     }
@@ -1244,6 +1260,12 @@ client_main(
 	    errExit("SSL_OptionSet SSL_ENABLE_DEFLATE");
     }
 
+    if (enableFalseStart) {
+	rv = SSL_OptionSet(model_sock, SSL_ENABLE_FALSE_START, PR_TRUE);
+	if (rv != SECSuccess)
+	    errExit("SSL_OptionSet SSL_ENABLE_FALSE_START");
+    }
+
     SSL_SetURL(model_sock, hostName);
 
     SSL_AuthCertificateHook(model_sock, mySSLAuthCertificate, 
@@ -1268,7 +1290,7 @@ client_main(
 
     if (!NoReuse) {
         remaining_connections = 1;
-	rv = launch_thread(do_connects, &addr, model_sock, 0);
+	rv = launch_thread(do_connects, &addr, model_sock, 0, socketDomain);
 	/* wait for the first connection to terminate, then launch the rest. */
 	reap_threads();
         remaining_connections = total_connections - 1 ;
@@ -1277,7 +1299,7 @@ client_main(
         active_threads  = PR_MIN(active_threads, remaining_connections);
 	/* Start up the threads */
 	for (i=0;i<active_threads;i++) {
-	    rv = launch_thread(do_connects, &addr, model_sock, i);
+	    rv = launch_thread(do_connects, &addr, model_sock, i, socketDomain);
 	}
 	reap_threads();
     }
@@ -1354,7 +1376,7 @@ main(int argc, char **argv)
  
 
     optstate = PL_CreateOptState(argc, argv,
-                                 "23BC:DNP:TUW:a:c:d:f:in:op:qst:uvw:z");
+                                 "23BC:DNP:TUW:a:c:d:f:gin:op:qst:uvw:z");
     while ((status = PL_GetNextOpt(optstate)) == PL_OPT_OK) {
 	switch(optstate->option) {
 
@@ -1384,6 +1406,8 @@ main(int argc, char **argv)
 
 	case 'f': fileName = optstate->value; break;
 
+	case 'g': enableFalseStart = PR_TRUE; break;
+
 	case 'i': ignoreErrors = PR_TRUE; break;
 
         case 'n': nickName = PL_strdup(optstate->value); break;
