@@ -58,9 +58,9 @@
 // forward declarations
 class   nsIAppShell;
 class   nsIToolkit;
-class   nsIFontMetrics;
-class   nsIRenderingContext;
-class   nsIDeviceContext;
+class   nsFontMetrics;
+class   nsRenderingContext;
+class   nsDeviceContext;
 struct  nsFont;
 class   nsIRollupListener;
 class   nsIMenuRollup;
@@ -117,8 +117,8 @@ typedef nsEventStatus (* EVENT_CALLBACK)(nsGUIEvent *event);
 #endif
 
 #define NS_IWIDGET_IID \
-  { 0xe5c2efd1, 0xfbae, 0x4a74, \
-    { 0xb2, 0xeb, 0xf3, 0x49, 0xf5, 0x72, 0xca, 0x71 } }
+  { 0xac809e35, 0x632c, 0x448d, \
+    { 0x9e, 0x34, 0x11, 0x62, 0x32, 0x60, 0x5e, 0xe6 } }
 
 /*
  * Window shadow styles
@@ -233,6 +233,29 @@ struct nsIMEUpdatePreference {
 struct IMEContext {
   PRUint32 mStatus;
 
+  /* Does the change come from a trusted source */
+  enum {
+    FOCUS_REMOVED       = 0x0001,
+    FOCUS_MOVED_UNKNOWN = 0x0002,
+    FOCUS_MOVED_BY_MOVEFOCUS = 0x0004,
+    FOCUS_MOVED_BY_MOUSE = 0x0008,
+    FOCUS_MOVED_BY_KEY = 0x0010,
+    FOCUS_MOVED_TO_MENU = 0x0020,
+    FOCUS_MOVED_FROM_MENU = 0x0040,
+    EDITOR_STATE_MODIFIED = 0x0080,
+    FOCUS_FROM_CONTENT_PROCESS = 0x0100
+  };
+
+  PRBool FocusMovedByUser() const {
+    return (mReason & FOCUS_MOVED_BY_MOUSE) || (mReason & FOCUS_MOVED_BY_KEY);
+  };
+
+  PRBool FocusMovedInContentProcess() const {
+    return (mReason & FOCUS_FROM_CONTENT_PROCESS);
+  };
+
+  PRUint32 mReason;
+
   /* The type of the input if the input is a html input field */
   nsString mHTMLInputType;
 
@@ -307,7 +330,7 @@ class nsIWidget : public nsISupports {
                       nsNativeWidget   aNativeParent,
                       const nsIntRect  &aRect,
                       EVENT_CALLBACK   aHandleEventFunction,
-                      nsIDeviceContext *aContext,
+                      nsDeviceContext *aContext,
                       nsIAppShell      *aAppShell = nsnull,
                       nsIToolkit       *aToolkit = nsnull,
                       nsWidgetInitData *aInitData = nsnull) = 0;
@@ -331,7 +354,7 @@ class nsIWidget : public nsISupports {
     virtual already_AddRefed<nsIWidget>
     CreateChild(const nsIntRect  &aRect,
                 EVENT_CALLBACK   aHandleEventFunction,
-                nsIDeviceContext *aContext,
+                nsDeviceContext *aContext,
                 nsIAppShell      *aAppShell = nsnull,
                 nsIToolkit       *aToolkit = nsnull,
                 nsWidgetInitData *aInitData = nsnull,
@@ -350,7 +373,7 @@ class nsIWidget : public nsISupports {
      * aContext The new device context for the view
      */
     NS_IMETHOD AttachViewToTopLevel(EVENT_CALLBACK aViewEventFunction,
-                                    nsIDeviceContext *aContext) = 0;
+                                    nsDeviceContext *aContext) = 0;
 
     /**
      * Accessor functions to get and set secondary client data. Used by
@@ -916,14 +939,11 @@ class nsIWidget : public nsISupports {
     virtual void UpdateThemeGeometries(const nsTArray<ThemeGeometry>& aThemeGeometries) = 0;
 
     /**
-     * Informs the widget about the region of the window that is partially
-     * transparent. Widgets should assume that the initial transparent
-     * region is empty.
+     * Informs the widget about the region of the window that is opaque.
      *
-     * @param aTransparentRegion the region of the window that is partially
-     * transparent.
+     * @param aOpaqueRegion the region of the window that is opaque.
      */
-    virtual void UpdateTransparentRegion(const nsIntRegion &aTransparentRegion) {};
+    virtual void UpdateOpaqueRegion(const nsIntRegion &aOpaqueRegion) {};
 
     /** 
      * Internal methods
@@ -936,7 +956,7 @@ class nsIWidget : public nsISupports {
     virtual void FreeNativeData(void * data, PRUint32 aDataType) = 0;//~~~
 
     // GetDeviceContext returns a weak pointer to this widget's device context
-    virtual nsIDeviceContext* GetDeviceContext() = 0;
+    virtual nsDeviceContext* GetDeviceContext() = 0;
 
     //@}
 
